/*
  

  This is an example of an HTTP server that is accessible
  via http://bellgirl.local/open URL thanks to mDNS responder.

  Instructions:
  - Update WiFi SSID and password as necessary.
  - Flash the sketch to the ESP8266 board
  - Install host software:
    - For Linux, install Avahi (http://avahi.org/).
    - For Windows, install Bonjour (http://www.apple.com/support/bonjour/).
    - For Mac OSX and iOS support is built in through Bonjour already.
  - Point your browser to http://esp8266.local, you should see a response.

 */

// NeoPixel Ring simple sketch (c) 2013 Shae Erisson
// released under the GPLv3 license to match the rest of the AdaFruit NeoPixel library
#include <Adafruit_NeoPixel.h>
#ifdef __AVR__
#include <avr/power.h>
#endif

#include <ESP8266WiFi.h>
#include <ESP8266mDNS.h>
#include <WiFiClient.h>


#define LED             2
#define LOCKED          14
#define RELAY           12
#define RECORDER        5

// Which pin on the Arduino is connected to the NeoPixels?
// On a Trinket or Gemma we suggest changing this to 1
#define STRIPPIN        13
// How many NeoPixels are attached to the Arduino?
#define NUMPIXELS       15

// When we setup the NeoPixel library, we tell it how many pixels, and which pin to use to send signals.
// Note that for older NeoPixel strips you might need to change the third parameter--see the strandtest
// example for more information on possible values.
Adafruit_NeoPixel pixels = Adafruit_NeoPixel(NUMPIXELS, STRIPPIN, NEO_GRB + NEO_KHZ800);

unsigned long timeDelay = 0;
bool doorLocked = true;         //false = unlocked ,true = locked

const char* ssid = "STU-NoInternet";
const char* password = "depot0510";

// TCP server at port 80 will respond to HTTP requests
WiFiServer server(80);

void setup(void)
{
  pinMode(LED, OUTPUT);
  pinMode(LOCKED, INPUT);
  pinMode(RELAY, OUTPUT);
  pinMode(RECORDER, OUTPUT);
  Serial.begin(115200);

  pixels.begin(); // This initializes the NeoPixel library.

  // Connect to WiFi network
  WiFi.begin(ssid, password);
  Serial.println("");

  // Wait for connection
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.print("Connected to ");
  Serial.println(ssid);
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());

  // Set up mDNS responder:
  // - first argument is the domain name, in this example
  //   the fully-qualified domain name is "esp8266.local"
  // - second argument is the IP address to advertise
  //   we send our IP address on the WiFi network
  if (!MDNS.begin("bellgirl2")) {
    Serial.println("Error setting up MDNS responder!");
    while (1) {
      ; //if setup mDNS error, it will stop here and cause the watch dog reset!
    }
  }
  Serial.println("mDNS responder started");

  // Start TCP (HTTP) server
  server.begin();
  Serial.println("TCP server started");

  // Add service to MDNS-SD
  MDNS.addService("http", "tcp", 80);

  doorStatusClosed();
}

void loop(void)
{
  if (WiFi.status() != WL_CONNECTED) {
    while (1); //if wifi connection error, it will stop here and cause the watch dog reset!
  }
  clientProcess();               // process the client requests
  checkDoorLockStatus();         //check the door lock status
}

void playVoiceRecord (void) {
  digitalWrite(RECORDER, HIGH);
  delay(4000);
  digitalWrite(RECORDER, LOW);
}

//This fuction just change the door status and the indicator not really open or close the door
void doorStatusOpen (void) {  
  doorLocked = false;     //the door status is open
  
  //setup LEDs to green
  digitalWrite(LED, HIGH);
  for (int i = 0; i < NUMPIXELS; i++) {

    // pixels.Color takes RGB values, from 0,0,0 up to 255,255,255
    pixels.setPixelColor(i, pixels.Color(0, 150, 0)); // Moderately bright green color.

  }
  pixels.show(); // This sends the updated pixel color to the hardware.

  playVoiceRecord();
}

//This fuction just change the door status and the indicator not really open or close the door
void doorStatusClosed (void) {
  doorLocked = true;    //the door is closed
  //setup LEDs to red
  digitalWrite(LED, LOW);
  for (int i = 0; i < NUMPIXELS; i++) {

    // pixels.Color takes RGB values, from 0,0,0 up to 255,255,255
    pixels.setPixelColor(i, pixels.Color(150, 0, 0)); // Moderately bright green color.

  }
  pixels.show(); // This sends the updated pixel color to the hardware.
}

void checkDoorLockStatus (void) {
  if (doorLocked == true) {      //if the door status is closed
    if (digitalRead(LOCKED) == false) {
      delay(100);                //delay recheck the sensor to avoid disturbance
      if (digitalRead(LOCKED) == false) {
        doorStatusOpen();
      }
    }
  } else {    //if the door status is open
    if (digitalRead(LOCKED) == true) {
      delay(100);
      if (digitalRead(LOCKED) == true) {
        doorStatusClosed();
      }
    }
  }
}

void clientProcess (void) {
  // Check if a client has connected
  WiFiClient client = server.available();
  if (!client) {
    return;
  }
  Serial.println("");
  Serial.println("New client");

  // Wait for data from client to become available
  while (client.connected() && !client.available()) {
    checkDoorLockStatus();
    delay(1);
  }

  // Read the first line of HTTP request
  String req = client.readStringUntil('\r');

  // First line of HTTP request looks like "GET /path HTTP/1.1"
  // Retrieve the "/path" part by finding the spaces
  int addr_start = req.indexOf(' ');
  int addr_end = req.indexOf(' ', addr_start + 1);
  if (addr_start == -1 || addr_end == -1) {
    Serial.print("Invalid request: ");
    Serial.println(req);
    return;
  }
  req = req.substring(addr_start + 1, addr_end);
  Serial.print("Request: ");
  Serial.println(req);
  client.flush();

  String s;
  if (req.indexOf("/open") != -1)
  {
    IPAddress ip = WiFi.localIP();
    String ipStr = String(ip[0]) + '.' + String(ip[1]) + '.' + String(ip[2]) + '.' + String(ip[3]);
    s = "HTTP/1.1 200 OK\r\nContent-Type: text/html\r\n\r\n<!DOCTYPE HTML>\r\n<html>The door is open! ";
    s += "   bellgirl ip :";
    s += ipStr;
    s += "</html>\r\n\r\n";
    Serial.println("Sending 200");

    //door lock pr
    if (checkPermission()) {
      //switch on relay
      digitalWrite(RELAY, HIGH);  //unlock the door
      doorLocked = false;
      doorStatusOpen();          //this function will delay(4000) for playing the voice record
      digitalWrite(RELAY, LOW);   //lock the door
    }
  }
  else
  {
    s = "HTTP/1.1 404 Not Found\r\n\r\n";
    Serial.println("Sending 404");
  }
  client.print(s);

  Serial.println("Done with client");

}


bool checkPermission (void) {
  return true;
  //you can add some permission check process
}
